/*******************************************************************************
 * Copyright (c) 2003, 2007 s IT Solutions AT Spardat GmbH .
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     s IT Solutions AT Spardat GmbH - initial API and implementation
 *******************************************************************************/

/*
 * Created on 04.11.2003
 *
 * 
 * 
 */
package at.spardat.xma.mdl.list;

import at.spardat.enterprise.fmt.IFmt;
import at.spardat.xma.mdl.IWModelClient;

/**
 * Client side interface of the widged model to represent a list of choises.
 * It adds the functionality of formatters to its super interface.
 * 
 * @author s2877
 */
public interface IListWMClient extends IListWM, IWModelClient {

    /**
     * Returns the associated Formatter or null if none is associated.
     * 
     * @return the attached formatter
     */
    IFmt getFmt();

    /**
     * Sets a new Formatter. 
     * 
     * @param formatter to be set. It must be compatible to the type of the entries. 
     *         For type T_STRING, any formatter may be set. For the other types, the type of the
     *         formatter must be compatible with the type specified in the superclass. 
     */
    void setFmt (IFmt formatter);

    /**
     * Provides a string that results from applying the formatter to the internally stored
     * representation of the selected entry.
     */
    public String getFormattedString ();
 
    /**
     * Returns the MANDATORY-property of the associated formatter. Returns false, if there
     * is no formatter associated.
     */
    public boolean isMandatory ();
    
    /**
     * If there is a formatter attached, this method clones the old formatter, changes
     * the MANDATORY-property of the formatter to the provided value, and sets the formatter.
     * If there is no formatter attached, this method does nothing.
     */
    public void setMandatory (boolean what);

    /**
     * @return if prefix search will be done case sensitive
     * @since 1.7.3
     */
    public boolean isPrefixSearchCaseSensitive();
    
    /**
     * determine if prefix seach has to be done case sensitive or not 
     * @param prefixSearchCaseSensitive
     * @since 1.7.3
     */
    public void setPrefixSearchCaseSensitive(boolean prefixSearchCaseSensitive);

    /**
     * @return if prefix search will be done on this list model
     * @since 1.8.1
     */
    public boolean isDoPrefixSearch();

    /**
     * determine if prefix search has to be done on this model 
     * @since 1.8.1
     */
    public void setDoPrefixSearch(boolean doPrefixSearch);
    
}
