/*******************************************************************************
 * Copyright (c) 2003, 2009 s IT Solutions AT Spardat GmbH .
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     s IT Solutions AT Spardat GmbH - initial API and implementation
 *******************************************************************************/

package at.spardat.xma.page;

import at.spardat.xma.mdl.IWModelClient;
import at.spardat.xma.mdl.NewModelEventParams;

/**
 * Container object used to hold the informations needed for delayed creation and attachment
 * of widgets for dynamically created widget models. This delays are needed if widget models
 * is dynamically created before <tt>initGUI()</tt> was called.
 *
 * @author gub
 * @since 2.1.0
 */
public class DynamicModelSet {
	/** dynamically created model */
	IWModelClient model;
	/** widgets corresponding to the model */
	ControlSet controls;
	/** parameters used to create the validator and the widgets for the model */
	NewModelEventParams params;

	/**
	 * Creates a DynamicModelSet and sets its model.
	 * @param model the dynamically created widget model.
	 */
	public DynamicModelSet(IWModelClient model) {
		this.model=model;
	}

	/**
	 * Returns the widget model where the data stored in this belongs to.
	 */
	public IWModelClient getModel() {
		return model;
	}

	/**
	 * Sets the model for this.
	 */
	public void setModel(IWModelClient model) {
		this.model = model;
	}

	/**
	 * Returns the controls corresponding to the model
	 */
	public ControlSet getControls() {
		return controls;
	}

	/**
	 * Sets the controls corresponding to the model
	 */
	public void setControls(ControlSet controls) {
		this.controls = controls;
	}

	/**
	 * Returns the parameters used to create the validator and the widgets for the model
	 */
	public NewModelEventParams getParams() {
		return params;
	}

	/**
	 * Sets the parameters used to create the validator and the widgets for the model
	 */
	public void setParams(NewModelEventParams params) {
		this.params = params;
	}


}
